import Tools.IO.*;                              // Eingaben

/*
 * Statistisch Auswertung von Zufallsfeldern.
 */
public class RandomArray
{
/*
 * Erzeugen eines Zufallsfeldes.
 * @param n Anzahl der Komponenten
 * @param M Obergrenze
 * @return Zufallsfeld
 */
  public static long[] randomArray( int n, int m)
  {
    // Erzeugen von n Zusallszahlen im Bereich [ 1, m]
    long[] x = new long[ n];
    for( int i = 0; i < x.length; i++)
      x[ i] = (long)( Math.random() * m + 1);
    return x;
  }

/*
 * Ausgabe eines Feldes
 * @param x Feld
 */
  public static void ausgabe( long[] x)
  {
    for( int i = 0; i < x.length; i++)
    {
      if( i % 10 == 0) System.out.println();
      System.out.print( "\t" + x[ i]);
    }
    System.out.println();
  }

/*
 * Sortieren eines Feldes
 * @param x unsortiertes Feld
 * @return sortiertes Feld
 */
  public static long[] bubbleSort( long[] x)
  {
    // Kopie erzeugen
    long[] y = new long[ x.length];
    for( int i = 0; i < y.length; i++) y[ i] = x[ i];
		
    // Bubble-Sort auf der Kopie
    for( int i = y.length; i > 1;)
    {
      int lastPos = 0;     // letzte unsortierte Komponente

      for( int j = 0; j < i - 1; j++)
      {
        // Reihenfolge benachbarter Komponenten
        if( y[ j] > y[ j + 1])
        {
          lastPos = j + 1;
          long tmp = y[ j];                   // Vertauschen
          y[j] = y[j+1];
          y[j+1] = tmp;
        }
      }
      i = lastPos;
    }
    return y;
  }

/*
 * Mittelwert eines Feldes.
 * @param x Feld
 * @return Mittelwert
 */
  public static double mittelWert( long[] x)
  {
    if( x.length == 0) return -1;              // Feld leer

    double m = .0;
    for( int i = 0; i < x.length; i++) m += x[ i];

    return m / x.length;
  }

/*
 * Varianz eines Feldes
 * @param x Feld
 * @return Varianz
 */
  public static double varianz( long[] x)
  {
    if( x.length == 0) return -1;              // Feld leer
    if( x.length == 1) return 0;           // Varianz ist 0

    double m = mittelWert( x);
    double v = .0;

    for(int i = 0; i < x.length; i++)
      v += ( m - x[ i]) * ( m - x[ i]);
    
    return v / ( x.length - 1);
  }

/*
 * Testprogramm,
 * erzeugt Zufallsfeld, sortiert es mittels BubbleSort,
 * ermittelt Mittelwert und Varianz.
 */
  public static void main(String[] args)
  {
    System.out.println();

/* ----------------------------------------------------- */
    // a) Erzeugen eines Zufallsfeldes und dessen Ausgabe

    // Eingabe der Feldlaenge und des Wertebereichs
    int n
    = IOTools.readInteger( "Anzahl der Komponenten? n = ");
    System.out.println( n);
    int m
    = IOTools.readInteger( "Wertebereich [ 1, m]? m = ");
    System.out.println( m);

    // Erzeugen und Ausgabe des Zufallsfeldes
    long[] zahlen = randomArray( n, m);
    System.out.println( "Unsortiert:");
    ausgabe( zahlen);

/* ----------------------------------------------------- */
    // b) Sortieren einer Kopie der Feldes mit Bubble-Sort

    // Sortieren und Ausgabe des Zufallsfeldes
    System.out.println( "Sortiert:");
    ausgabe( bubbleSort( zahlen));

/* ----------------------------------------------------- */
    // c) Berechnung Mittelwert, Varianz und Ausgaben

    // Berechnung und Ausgabe des Mittelwertes
    System.out.println( "Mittelwert\t: " + mittelWert( zahlen));

    // Berechnung und Ausgabe der Varianz
    System.out.println( "Varianz\t: " + varianz( zahlen));
  }
}

/*
Anzahl der Komponenten? n = 100
Wertebereich [ 1, m]? m = 50
Unsortiert:

	16	41	16	47	26	47	46	2	8	5
	45	2	23	32	42	27	38	19	43	28
	8	17	27	16	33	6	6	29	37	42
	21	17	8	13	15	4	29	37	39	36
	41	27	13	24	23	5	17	3	4	32
	13	7	22	7	26	23	4	35	42	19
	1	45	13	30	22	15	26	31	29	48
	50	49	49	21	31	13	34	40	7	1
	5	24	13	37	46	8	46	8	41	35
	2	41	31	39	38	40	26	22	1	39
Sortiert:

	1	1	1	2	2	2	3	4	4	4
	5	5	5	6	6	7	7	7	8	8
	8	8	8	13	13	13	13	13	13	15
	15	16	16	16	17	17	17	19	19	21
	21	22	22	22	23	23	23	24	24	26
	26	26	26	27	27	27	28	29	29	29
	30	31	31	31	32	32	33	34	35	35
	36	37	37	37	38	38	39	39	39	40
	40	41	41	41	41	42	42	42	43	45
	45	46	46	46	47	47	48	49	49	50
Mittelwert	: 24.77
Varianz	: 215.04757575757583

Anzahl der Komponenten? n = 100
Wertebereich [ 1, m]? m = 20
Unsortiert:

	16	18	8	8	19	6	18	1	19	16
	3	17	19	8	1	12	4	2	20	15
	4	12	5	9	12	13	3	1	16	1
	16	18	4	9	18	6	7	1	3	14
	1	9	15	4	10	9	11	11	18	16
	17	11	3	2	18	11	16	15	15	13
	5	16	13	4	13	2	20	2	17	17
	10	10	4	8	9	8	11	3	15	16
	20	4	16	15	9	18	11	12	10	5
	7	7	9	8	4	11	5	15	13	19
Sortiert:

	1	1	1	1	1	1	2	2	2	2
	3	3	3	3	3	4	4	4	4	4
	4	4	4	5	5	5	5	6	6	7
	7	7	8	8	8	8	8	8	9	9
	9	9	9	9	9	10	10	10	10	11
	11	11	11	11	11	11	12	12	12	12
	13	13	13	13	13	14	15	15	15	15
	15	15	15	16	16	16	16	16	16	16
	16	16	17	17	17	17	18	18	18	18
	18	18	18	19	19	19	19	20	20	20
Mittelwert	: 10.48
Varianz	: 33.262222222222235

Anzahl der Komponenten? n = 100
Wertebereich [ 1, m]? m = 10
Unsortiert:

	3	4	2	8	3	4	8	2	1	4
	10	1	9	9	10	1	5	2	7	6
	9	10	1	7	4	3	3	10	5	10
	5	6	5	3	8	9	7	8	7	3
	8	9	3	8	1	9	9	10	8	5
	6	9	10	9	3	4	9	10	3	2
	3	4	3	10	7	8	10	1	3	10
	8	5	5	8	3	3	7	3	4	1
	4	9	6	3	9	1	3	3	8	9
	3	5	9	1	3	7	6	8	9	1
Sortiert:

	1	1	1	1	1	1	1	1	1	1
	2	2	2	2	3	3	3	3	3	3
	3	3	3	3	3	3	3	3	3	3
	3	3	3	3	4	4	4	4	4	4
	4	4	5	5	5	5	5	5	5	5
	6	6	6	6	6	7	7	7	7	7
	7	7	8	8	8	8	8	8	8	8
	8	8	8	8	9	9	9	9	9	9
	9	9	9	9	9	9	9	9	9	10
	10	10	10	10	10	10	10	10	10	10
Mittelwert	: 5.7
Varianz	: 8.999999999999998
*/
